-- M.A.I.D.  (Mining And Inventory Daemon)
--
-- Assembled and written by igrok
-- Cargo timer adopted from Timer by Chocolateer
--
--

MAID.time = {}
MAID.time.timers = {}
MAID.time.startTimes = {}

-- Cargo times out 15 minutes after it's been dropped.
MAID.time.duration = 60 * 15 * 1000


-- start timer for cargo item that was dropped (15 minutes)
function MAID.time:startTimer(cargoitem)
    -- make sure timer isn't already running
    local itemTime = MAID.time:remaining(cargoitem)

    if itemTime == nil or itemTime == 0 then
        MAID:debugPrint("MAID: starting cargo timer for " .. cargoitem)

        -- initialize the timer object
        if MAID.time.timers[cargoitem] == nil then
            MAID.time.timers[cargoitem] = Timer()
        else
            MAID.time.timers[cargoitem]:Kill()
        end

        -- start running the timer
        MAID.time.startTimes[cargoitem] = gkmisc.GetGameTime()
        MAID.time.timers[cargoitem]:SetTimeout(MAID.time.duration, function()
                MAID.time:complete(cargoitem)
            end)
    else
        MAID:debugPrint("MAID: not starting timer for " .. cargoitem .. " - timer already running...")
    end
end


-- stop cargo timer for item
function MAID.time:stopTimer(cargoitem)
    -- make sure timer is running
    if MAID.time.timers[cargoitem] and MAID.time.timers[cargoitem]:IsActive() then
        MAID.time.timers[cargoitem]:Kill()
        MAID:debugPrint("MAID: killed timer for " .. cargoitem)
    else
        MAID:debugPrint("MAID: unable to kill timer for " .. cargoitem .. " - timer not running.")
    end
end


-- return the amount of time cargo has been floating
-- returns nil if the timer has not been started
function MAID.time:elapsed(cargoitem)
    if MAID.time.startTimes[cargoitem] > 0 then
        return gkmisc.DiffTime(MAID.time.startTimes[cargoitem], gkmisc.GetGameTime())
    else
        return nil
    end
end


-- return the amount of time left on the timer
-- returns nil if the timer is not running
function MAID.time:remaining(cargoitem)
    if MAID.time.timers[cargoitem] and MAID.time.timers[cargoitem]:IsActive() then
        return MAID.time.duration - gkmisc.DiffTime(MAID.time.startTimes[cargoitem], gkmisc.GetGameTime())
    else
        return nil
    end
end


--Convert HH:MM:SS notation to msec
function MAID.time:str2msec(str)
    local time_tbl = unspickle('\"'..str:gsub(":", "\",\"")..'\"') or {}
    local total_time = 0 --in msecs
    local mult = 60*60*1000
    
    if table.getn(time_tbl) > 1 then
        for _,t in ipairs(time_tbl) do
            total_time = total_time + tonumber(t)*mult
            mult = mult/60
        end
        
        return total_time
    else return tonumber(str)*60*1000 end
end


--Converts msec to string in H:MM:SS notation
function MAID.time:msec2str(msec)
    local time = MAID.time:msec2table(msec)
    local str = ''
    
    -- if less than 10, we pad with a zero
    time.seconds = math.floor(time.seconds)
    if time.minutes < 10 then
        time.minutes = '0' .. time.minutes
    end
    if time.seconds < 10 then
        time.seconds = '0' .. time.seconds
    end

    str = str .. time.hours .. 'h:'
    str = str .. time.minutes .. 'm:'
    str = str .. time.seconds .. 's'

    return str
end


--Convert msec to table containing hours, minutes, seconds
function MAID.time:msec2table(msec)
    msec = tonumber(msec) or 0
    local time = {hours=0, minutes=0, seconds=0, total=msec}

    local const = 1000*60*60
    if msec >= const then
        time.hours = math.floor(msec/const)
        msec = msec%const
    end

    const = 1000*60
    if msec >= const then
        time.minutes = math.floor(msec/const)
        msec = msec%const
    end

    time.seconds = msec/1000

    return time
end


function MAID.time:complete(cargoitem)
    MAID:print("M.A.I.D.: cargo has expired: " .. cargoitem)
    MAID:playSound('timer_end')
end
